#include <htc.h>
#include "delay.h"
#include "main.h"

__CONFIG( INTCLK & WDTDIS & PWRTDIS & MCLRDIS & UNPROTECT & \
		  DUNPROTECT & BORDIS & IESODIS & FCMDIS & LVPDIS & \
		  DEBUGDIS );

unsigned char direction = 0;	// Variable that specifies direction of motor next
unsigned char adhigh = 0;		// Holds 2 highest bits of A/D
unsigned char adlow = 0;		// Holds 8 lowest bits of A/D
unsigned int AtoD = 0;			// Holds full A/D value
//unsigned int calread = 0;		// Read calibration value
//unsigned int caltheor = 0x007A;	// Theoretical calibration value
//unsigned char diff = 0;			// Difference between the two
//unsigned char higher = 0;		// If voltage reads higher than theoretical, this is set
unsigned int up = 0x0099;		// Voltage trimpot reads when screen is up
unsigned int down = 0x0369;		// Voltage trimpot reads when screen is down
unsigned char tilt = 0x00;		// Tilt position LCD is at
unsigned int tiltpos1 = 0x00C9;	// Voltage trimpot reads when screen is at tilt position 1
unsigned int tiltpos2 = 0x0FC;	// Voltage trimpot reads when screen is at tilt position 2
unsigned int tiltpos3 = 0x0131;	// Voltage trimpot reads when screen is at tilt position 3
unsigned int tiltpos4 = 0x015C;	// Voltage trimpot reads when screen is at tilt position 1
int i;							// Used for FOR loops

//***************************
// Main function of program
//***************************
void main ()
{
	init();						// Initialize things
//	calibrate();
while (1)						// Loop forever (we only want to run init once)
{
	while(RB5)					// RB5 is 1 when car is on (ACC is +12v)
	{
		PORTC = 0x11;			// Brake motor
		
		if (!RB0)				// RB0 is 0 when open button is pressed
		{
			direction = !direction;	// Change direction variable
		
			if (direction == 0)	// Screen is currently closed
			{
				open();			// Run open function
			}
			if (direction != 0)	// Screen is currently open
			{
				close();		// Run close function
			}
		}
		
		if (direction == 0 & !RB1)	// RB1 is 0 when tilt button is pressed
		{							// Also, tilt should only work when screen is open
			tilt++;					// Increment tilt position variable
			if (tilt == 0x01)		// First time tilt has been pressed
			{
				tilt1();			// Tilt to first position
			}
			else if (tilt == 0x02)	// Second time tilt has been pressed
			{
				tilt2();			// Tilt to second position
			}
			else if (tilt == 0x03)	// Third time tilt has been pressed
			{
				tilt3();			// Tilt to third position
			}
			else open();			// Fourth time tilt has been pressed, go back to open
			
		}		
	}
	for (i = 0; i < 4; i++)
	{
		DelayMs(250);
	}
	close();
	ADON = 0;
	ANSEL = 0x00;
	TRISA = 0x00;
	PORTA = 0x00;
	//TRISB = 0x00;
	WPUB = 0x00;
	PORTB = 0x00;
	PORTC = 0x00;

	RBIF = 0;
	SLEEP();
	ADON = 1;
	ANSEL = 0x01;
	TRISA = 0x01;
	//TRISB = 0x23;
	WPUB = 0x03;
	PORTA = 0x04;
	for (i = 0; i < 4; i++)
	{
		DelayMs(250);
	}
		if (direction == 0)
		{
			if (tilt == 0x01)
			{
				tiltfc1();
			}
			else if (tilt == 0x02)
			{
				tiltfc2();
			}
			else if (tilt == 0x03)
			{
				tiltfc3();
			}
			else open();
			
		}
}		
	//main();		
}

void open (void)
{
	PORTC = 0x13;
	while (AtoD >= up)
	{
		grabAD();
	}
	PORTC = 0x11;
	DelayMs(125);
	tilt = 0x00;
	return;
}

void close (void)
{
	PORTC = 0x15;
	while (AtoD <= down)
	{
		grabAD();
	}
	PORTC = 0x11;
	DelayMs(125);
	return;
}

void tilt1 (void)
{
	PORTC = 0x15;
	while (AtoD <= tiltpos1)
	{
		grabAD();
	}
	PORTC = 0x11;
	DelayMs(125);
	return;
}

void tilt2 (void)
{
	PORTC = 0x15;
	while (AtoD <= tiltpos2)
	{
		grabAD();
	}
	PORTC = 0x11;
	DelayMs(125);
	return;
}

void tilt3 (void)
{
	PORTC = 0x15;
	while (AtoD <= tiltpos3)
	{
		grabAD();
	}
	PORTC = 0x11;
	DelayMs(125);
	return;
}

void tiltfc1 (void)
{
	PORTC = 0x13;
	while (AtoD >= tiltpos2)
	{
		grabAD();
	}
	PORTC = 0x11;
	DelayMs(125);
	return;
}

void tiltfc2 (void)
{
	PORTC = 0x13;
	while (AtoD >= tiltpos3)
	{
		grabAD();
	}
	PORTC = 0x11;
	DelayMs(125);
	return;
}

void tiltfc3 (void)
{
	PORTC = 0x13;
	while (AtoD >= tiltpos4)
	{
		grabAD();
	}
	PORTC = 0x11;
	DelayMs(125);
	return;
}

void init (void)
{
	ANSEL = 0x01;		// Set AN0 analog, rest
	ANSELH = 0x00;		// of the pins digital
	PORTA = 0x04;		// Initialize port A
	PORTB = 0x00;		// Initialize port B
	PORTC = 0x10;		// Initialize port C
	TRISA = 0x01;		// Set RA0/AN0 as input
	TRISB = 0x23;		// Set RB0, RB1, RB3 as inputs
	TRISC = 0x00;		// Set all port C as output
	WPUB = 0x03;		// Enable weak pull-up
	RBPU = 0;			// Global pull-up enable
	ADCON1 = 0x80;		// ADC output is right justified
	direction = 0x00;	// Initialize direction as 0
	caltheor = 0x007A;
	up = 0x0099;
	down = 0x0369;
	tiltpos1 = 0x00C9;
	tiltpos2 = 0x00FC;
	tiltpos3 = 0x0131;
	tiltpos4 = 0x015F;
	IOCB = 0x20;		// Interrupt on change for RB3
	INTCON = 0x08;		// Enable IOC, disable everything else
	SWDTEN = 0;			// Disable WDT
	GIE = 0;
	C1ON = 0;
	C2ON = 0;
	TMR1ON = 0;
	TMR2ON = 0;
	T1OSCEN = 0;
	VREN = 0;
	
	return;
}

void convert (void)
{
	if (GODONE == 0)		// Continue when conversion complete
	{
		adhigh = ADRESH;	// First 2 bits of conversion
		adlow = ADRESL;		// Last 8 bits of conversion
		AtoD = ((adhigh << 8) + adlow);		//Full conversion
		return;
	}
	else convert();			// Go back if not ready yet
}

void grabAD (void)
{
	ADCON0 = 0x41;			// Set A/D to RA0
//	DelayMs(5);				// Delay for 5 ms
	GODONE = 1;				// Start A to D conversion
	convert();				// Run function
/*	if (higher == 1)
	{
		AtoD = AtoD - diff;	// Calibrate result
	}
	else AtoD = AtoD + diff;// Calibrate result
*/
//	DelayMs(5);
}

void gotosleep (void)
{
	SLEEP();
	return;
}	

/*
Calibration code that gives wacky results


void calibrate (void)
{
	ADCON0 = 0x7D;			// Set A/D to 0.6v reference
	DelayMs(5);
	GODONE = 1;				// Start A/D
	convert();				// Run Conversion
	calread = AtoD;			// Set value as read calibration value
	if (calread <= caltheor)
	{						// Is read value below theoretical?
		diff = caltheor - calread;	// Difference is positive
		higher = 0;			// Lower, so higher is not set
	}
	else
	{						// Read value is above theoretical
		diff = calread - caltheor;	// Difference would be negative, so flip it
		higher = 1;			// Set higher bit
	}
	ADCON0 = 0x41;			// Set A/D to RA0
	return;
}

*/