#include <htc.h>
#include "delay.h"
#include "main.h"

__CONFIG( INTCLK & WDTDIS & PWRTDIS & MCLREN & UNPROTECT & \
		  DUNPROTECT & BORDIS & IESODIS & FCMDIS & LVPDIS & \
		  DEBUGDIS );

unsigned char direction = 0;			// Variable that specifies direction of motor next
unsigned char adhigh = 0;				// Holds 2 highest bits of A/D
unsigned char adlow = 0;				// Holds 8 lowest bits of A/D
unsigned int AtoD = 0;					// Holds full A/D value
unsigned int up = 0x0099;				// Voltage trimpot reads when screen is up
unsigned int down = 0x0330;				// Voltage trimpot reads when screen is down
unsigned char tilt = 0x00;				// Tilt position LCD is at
unsigned int tiltpos1 = 0x00CD;			// Voltage trimpot reads when screen is at tilt position 1
unsigned int tiltpos2 = 0x0FC;			// Voltage trimpot reads when screen is at tilt position 2
unsigned int tiltpos3 = 0x0131;			// Voltage trimpot reads when screen is at tilt position 3
unsigned int tiltfcpos1 = 0x00EB;		// Voltage trimpot reads when screen is at tilt position 1 from close
unsigned int tiltfcpos2 = 0x010D;		// Voltage trimpot reads when screen is at tilt position 2 from close
unsigned int tiltfcpos3 = 0x013E;		// Voltage trimpot reads when screen is at tilt position 3 from close
int i;									// Used for FOR loops

//***************************
// Main function of program
//***************************
void main ()
{
	init();								// Initialize things
	while (1)							// Loop forever (we only want to run init once)
	{
		while(RB5)						// RB5 is 1 when car is on (ACC is +12v)
		{
			PORTC = 0x11;				// Brake motor

			if (!RB0)					// RB0 is 0 when open button is pressed
			{
				direction = !direction;	// Change direction variable

				if (direction == 0)		// Screen is currently closed
				{
					open();				// Run open function
				}
				if (direction != 0)		// Screen is currently open
				{
					close();			// Run close function
				}
			}

			if (direction == 0 & !RB1)	// RB1 is 0 when tilt button is pressed
			{							// Also, tilt should only work when screen is open
				tilt++;					// Increment tilt position variable
				if (tilt == 0x01)		// First time tilt has been pressed
				{
					tilt1();			// Tilt to first position
				}
				else if (tilt == 0x02)	// Second time tilt has been pressed
				{
					tilt2();			// Tilt to second position
				}
				else if (tilt == 0x03)	// Third time tilt has been pressed
				{
					tilt3();			// Tilt to third position
				}
				else open();			// Fourth time tilt has been pressed, go back to open
			}
		}
		car_off();						// Car has been turned off, run car_off function
	}
}

//***************************
// Open the LCD screen
//***************************
void open (void)
{
	PORTC = 0x13;						// Turn motor on clockwise
	while (AtoD >= up)					// Check A/D converter for trimpot value,
	{									// continue to run until it hits "up" voltage
		grabAD();						// Run A/D acquisiton/conversion
	}
	PORTC = 0x11;						// Brake motor
	DelayMs(125);						// Wait approx. 1/8 second
	tilt = 0x00;						// Set tilt variable to 0
	return;
}

//***************************
// Close the LCD screen
//***************************
void close (void)
{
	PORTC = 0x15;						// Turn motor on counterclockwise
	while (AtoD <= down)				// Check A/D converter for trimpot value,
	{									// continue to run until it hits "down" voltage
		grabAD();						// Run A/D acquisition/conversion
	}
	PORTC = 0x11;						// Brake motor
	DelayMs(125);						// Wait approx. 1/8 second
	return;
}

//***************************
// Tilt LCD to position 1
//***************************
void tilt1 (void)
{
	PORTC = 0x15;						// Turn motor on counterclockwise
	while (AtoD <= tiltpos1)			// Check A/D converter for trimpot value,
	{									// continue to run until it hits "tiltpos1" voltage
		grabAD();						// Run A/D acquisition/conversion
	}
	PORTC = 0x11;						// Brake motor
	DelayMs(125);						// Wait approx. 1/8 second
	return;
}

//***************************
// Tilt LCD to position 2
//***************************
void tilt2 (void)
{
	PORTC = 0x15;						// Turn motor on counterclockwise
	while (AtoD <= tiltpos2)			// Check A/D converter for trimpot value,
	{									// continue to run until it hits "tiltpos2" voltage
		grabAD();						// Run A/D acquisition/conversion
	}
	PORTC = 0x11;						// Brake motor
	DelayMs(125);						// Wait approx. 1/8 second
	return;
}

//***************************
// Tilt LCD to position 3
//***************************
void tilt3 (void)
{
	PORTC = 0x15;						// Turn motor on counterclockwise
	while (AtoD <= tiltpos3)			// Check A/D converter for trimpot value,
	{									// continue to run until it hits "tiltpos3" voltage
		grabAD();						// Run A/D acquisition/conversion
	}
	PORTC = 0x11;						// Brake motor
	DelayMs(125);						// Wait approx. 1/8 second
	return;
}

//***********************************
// Tilt LCD to position 1 from close
//***********************************
void tiltfc1 (void)
{
	PORTC = 0x13;						// Turn motor on clockwise
	while (AtoD >= tiltfcpos1)			// Check A/D converter for trimpot value,
	{									// continue to run until it hits "tiltfcpos1" voltage
		grabAD();						// Run A/D acquisition/conversion
	}
	PORTC = 0x11;						// Brake motor
	DelayMs(125);						// Wait approx. 1/8 second
	return;
}

//***********************************
// Tilt LCD to position 2 from close
//***********************************
void tiltfc2 (void)
{
	PORTC = 0x13;						// Turn motor on clockwise
	while (AtoD >= tiltfcpos2)			// Check A/D converter for trimpot value,
	{									// continue to run until it hits "tiltfcpos2" voltage
		grabAD();						// Run A/D acquisition/conversion
	}
	PORTC = 0x11;						// Brake motor
	DelayMs(125);						// Wait approx. 1/8 second
	return;
}

//***********************************
// Tilt LCD to position 3 from close
//***********************************
void tiltfc3 (void)
{
	PORTC = 0x13;						// Turn motor on clockwise
	while (AtoD >= tiltfcpos3)			// Check A/D converter for trimpot value,
	{									// continue to run until it hits "tiltfcpos3" voltage
		grabAD();						// Run A/D acquisition/conversion
	}
	PORTC = 0x11;						// Brake motor
	DelayMs(125);						// Wait approx. 1/8 second
	return;
}

//***************************
// Initialize microcontroller
//***************************
void init (void)
{
	ANSEL = 0x01;						// Set AN0 analog, rest
	ANSELH = 0x00;						// of the pins digital
	PORTA = 0x04;						// Initialize port A
	PORTB = 0x00;						// Initialize port B
	PORTC = 0x10;						// Initialize port C
	TRISA = 0x01;						// Set RA0/AN0 as input
	TRISB = 0x23;						// Set RB0, RB1, RB5 as inputs
	TRISC = 0x00;						// Set all port C as output
	WPUB = 0x03;						// Enable weak pull-up for RB<1:0>
	RBPU = 0;							// Global pull-up enable
	ADCON1 = 0x80;						// ADC output is right justified
	direction = 0x00;					// Initialize direction as 0
	up = 0x0099;						// Voltage trimpot reads when screen is up
	down = 0x0330;						// Voltage trimpot reads when screen is down
	tiltpos1 = 0x00CD;					// Voltage trimpot reads when screen is at tilt position 1
	tiltpos2 = 0x00FC;					// Voltage trimpot reads when screen is at tilt position 2
	tiltpos3 = 0x0131;					// Voltage trimpot reads when screen is at tilt position 3
	tiltfcpos1 = 0x00EB;				// Voltage trimpot reads when screen is at tilt position 1 from close
	tiltfcpos2 = 0x010D;				// Voltage trimpot reads when screen is at tilt position 2 from close
	tiltfcpos3 = 0x013E;				// Voltage trimpot reads when screen is at tilt position 3 from close
	IOCB = 0x20;						// Interrupt on change for RB5
	INTCON = 0x08;						// Enable IOC, disable everything else
	SWDTEN = 0;							// Disable Watchdog Timer
	GIE = 0;							// Turn off global interrupts
	C1ON = 0;							// Disable Comparator C1
	C2ON = 0;							// Disable Comparator C2
	TMR1ON = 0;							// Disable Timer1
	TMR2ON = 0;							// Disable Timer2
	T1OSCEN = 0;						// Disable LP oscillator
	VREN = 0;							// Disable CVRef circuit
	FVREN = 0;							// Disable fixed voltage reference
	return;
}

//***************************
// Acquire A/D value
//***************************
void grabAD (void)
{
	ADCON0 = 0x41;						// Set A/D to RA0
	GODONE = 1;							// Start A to D conversion
	convert();							// Run convert function
	return;
}

//***************************
// Convert A/D value
//***************************
void convert (void)
{
	if (GODONE == 0)					// Only continue when acquisition complete
	{
		adhigh = ADRESH;				// adhigh gets first 2 bits of conversion
		adlow = ADRESL;					// adlow gets last 8 bits of conversion
		AtoD = ((adhigh << 8) + adlow);	// AtoD gets full conversion value
		return;
	}
	else convert();						// Re-run if not ready yet
}

//***************************
// Car has been turned off
//***************************
void car_off (void)
{
	for (i = 0; i < 8; i++)				// Wait approx. 2 seconds
	{
		DelayMs(250);
	}
	close();							// Close the screen
	ADON = 0;							// Turn A/D converter off
	ANSEL = 0x00;						// Make all pins digital
	TRISA = 0x00;						// Make port A all output
	PORTA = 0x00;						// Turn all port A off
	WPUB = 0x00;						// Turn Weak pull-up off
	TRISB = 0x20;						// Make port B all output except RB5
	PORTB = 0x00;						// Turn all port B off (RB5 still input)
	PORTC = 0x00;						// Turn all port C off
	RBIF = 0;							// Clear port B interrupt on change flag
	SLEEP();							// Go to sleep to save power

/* 
 * At this point the microcontroller is consuming minimum
 * power, the only thing that will wake it up is an interrupt on
 * change set for RB5 when ACC is high, and instructions will continue 
 */   
	ADON = 1;							// Turn A/D converter on
	ANSEL = 0x01;						// Make RA0/AN0 analog
	TRISA = 0x01;						// Make RA0/AN0 an input
	PORTA = 0x04;						// Set RA2 high
	WPUB = 0x03;						// Turn weak pull-up on for RB<1:0>
	TRISB = 0x23;						// Make RB5, RB<1:0> input
	for (i = 0; i < 8; i++)				// Wait approx. 2 seconds
	{
		DelayMs(250);
	}
	if (direction == 0)					// If screen was open when car
	{									// was turned off
		if (tilt == 0x01)				// If screen was tilted in
		{								// first position
			tiltfc1();					// Run tilt from close 1 function
		}
		else if (tilt == 0x02)			// If screen was tilted in
		{								// second position
			tiltfc2();					// Run tilt from close 2 function
		}
		else if (tilt == 0x03)			// If screen was tilted in
		{								// third position
			tiltfc3();					// Run tilt from close 3 function
		}
		else open();					// Screen was fully open,
	}									// run open function
	return;
}